// smtpsend.c written by Cronos 09/01/2000

#include <winsock.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#define MAILBUFFSIZE 10000
#define SMTP_PORT 25

struct sockaddr_in A;
WSADATA wsaData;
int i;
struct hostent *s;
char R[MAILBUFFSIZE+1];

int init_winsock(WORD version,BOOL verbose)
{  int err;
	if(verbose) printf("Initialising Winsock\n");
	err = WSAStartup(version, &wsaData);
	if(err)
	{	printf("Could not initialise Winsock\n");
		if(verbose) printf("Error code:%d\n",WSAGetLastError());
	}
	return err;
}

void exit_winsock(void)
{	WSACleanup();
}

unsigned long hostlookupaddr(char *target,BOOL verbose)
{	unsigned long iadr;
	s=NULL;
	iadr=inet_addr(target);
	if(iadr==INADDR_NONE)
	{	if(verbose) printf("Trying gethostbyname\n");
		s=gethostbyname(target);
		if(s==NULL)
		{	printf("No entry for IP...(Error Code:%d)\n",WSAGetLastError());
			return 0;
		}
		return ((unsigned long *)(s->h_addr_list[0]))[0];
	}
	return iadr;
}

void helpscreen(void)
{	printf("\nSMTPSend by Cronos - help\n");
	printf("-------------------------\n");
	printf("SMTPSend attempts to send an email via a given smtp server without adding any messages to it and with minimal authorisation.\n");
	printf("The  start of the message should have the following three lines:\nMAIL FROM: <an email address>\nRCPT TO: <another email address>\nDATA\n");
	printf("Then add whatever other fields you want, followed by the email. On the last line there should be a single period '.' to mark the end of the email.\n");
	printf("The only messages that this program sends to the smtp server are 'HELO' and 'QUIT'.\n");
   printf("The email from address may be forged but the server might check domains and/or reverse dns lookup and disallow the send.\n");
	printf("Options:\n");
	printf("-h These help options\n");
	printf("-v Verbose mode (more messages)\n");
	printf("Example: smtpsend -v mailsvr.mail.com myemail.txt\n");
	printf("Example: smtpsend 128.200.34.45 email.txt\n");
}

int getsocket(int *sockn,BOOL verbose)
{  if(verbose) printf("Getting socket\n");
	if ((*sockn = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
	{	printf("Failed to get socket\n");
		if(verbose) printf("Errorcode: %d\n", WSAGetLastError());
		return -1;
	}
	return 0;
}

int opensocket(int sock, unsigned long server, short port,BOOL verbose)
{	struct sockaddr_in blah;

	memset((char *)&blah,'0', sizeof(blah));

	blah.sin_family=AF_INET;
	blah.sin_addr.s_addr=server;
	blah.sin_port=htons(port);

	if(verbose)
	{	printf("Mailserver resolved to : %s\n",inet_ntoa(blah.sin_addr));
		printf("Connecting....\n");
	}

	if (connect(sock,(struct sockaddr *)&blah,sizeof(blah))==SOCKET_ERROR)
	{ 	printf("Error:unable to connect(no mail service ?)\n");
		closesocket(sock);
		return(-4);
	}

	if(verbose)printf("Connected to [%s:%d].\n",inet_ntoa(blah.sin_addr),port);
	return 0;
}

int getresponse(int sockn,char *buff,int buffsize)
{  int i;
	i=recv(sockn,buff,buffsize,0);
	if(!i)
	{	printf("Server closed connection\n");
		return -1;
	}
	if(i==SOCKET_ERROR)
	{	printf("Socket Error returned:%d\n",WSAGetLastError());
		return -2;
	}
   buff[i]=0;
	printf("> %s",buff);
	if(!strncmpi(buff,"220",3)) return 0;			// ready
	if(!strncmpi(buff,"221",3)) return 0;        // closing
	if(!strncmpi(buff,"250",3)) return 0;			// ok
	if(!strncmpi(buff,"251",3)) return 0;        // will forward
	if(!strncmpi(buff,"354",3)) return 0;        // start data
	if(!strncmpi(buff,"421",3)) return -1;      	// closing transmit channel
	if(!strncmpi(buff,"45",2)) return -1;      	// action not taken
	if(!strncmpi(buff,"50",2)) return -1;       	// error reply
	if(!strncmpi(buff,"55",2)) return -1;        // unavailable/abort/fail
	return 1;
}

int sendline(int sockn, char *buff)
{  int i;
	printf("%s",buff);
	i=send(sockn,buff,strlen(buff),0);
	if(i==SOCKET_ERROR) return -1;
	return 0;
}

int sendfile(int sockn, char *fname)
{  int i;
	FILE *infile;
	BOOL hdrdone;
	if ((infile=fopen(fname,"rt"))==NULL)
	{ 	printf("Can't Open : %s\n",fname);
		return -1;
	}
	printf("Sending file\n");
	hdrdone=FALSE;
	while(!feof(infile))
	{ 	if((fgets(R,MAILBUFFSIZE-2,infile))==NULL)break;
		i=sendline(sockn,R);
		if(i==-1)
		{  fclose(infile);
			return -1;
		}
		if(!hdrdone)
		{	if(!strncmpi(R,"DATA",4)) hdrdone=TRUE;
			if(getresponse(sockn,R,MAILBUFFSIZE)<0)
			{	fclose(infile);
				return -1;
			}
		}
	}
	fclose(infile);
	return 0;
}

int main(int argc,char *argv[])
{	char *errv,*arv,*opts;
	int argn,mailnum,oc,textnum,mailsock;
	BOOL verb,hp;
	unsigned long padr;

	if (argc < 2)
	{	errv=argv[0];
		arv=errv;
		while(*errv)
		{	if(*errv=='\\') arv=errv+1;
			errv++;
		}
		printf("Usage: %s -[hv] <mailserver> <textfile>\n",arv);
		exit(-1);
	}
	verb=FALSE;
	hp=FALSE;
	argn=1;
	mailnum=0;
	textnum=0;
	while(argn<argc)
	{	opts=argv[argn];
		if((opts[0]!='-')&&(!mailnum)) mailnum=argn;
		else if((opts[0]!='-')&&(!textnum)) textnum=argn;
		else if(opts[0]=='-')
		{	// parse options.
			oc=1;
			while(opts[oc])
			{  switch(opts[oc])
				{	case 'h':
					case 'H':
						hp=TRUE;
						break;
					case 'v':
					case 'V':
						verb=TRUE;
						break;
					default:
						printf("Unknown option:%c\nUse -h for help\n",opts[oc]);
						exit(-1);
				}
				oc++;
			}
		}
		else
		{	printf("Mailserver set as:%s\n",argv[mailnum]);
			printf("Textfile set as  :%s\n",argv[textnum]);
			printf("Found %s in command line as well\n",argv[argn]);
			printf("Use -h for help\n");
			exit(-1);
		}
		argn++;
	}

	if(hp)
	{	helpscreen();
		if((!mailnum)||(!textnum)) exit(0);
	}

	if(!mailnum)
	{	printf("No mailserver found in command line\nUse -h for help\n");
		exit(-1);
	}

	if(!textnum)
	{	printf("No textfile found in command line\nUse -h for help\n");
		exit(-1);
	}

	if(verb)
	{	printf("SMTPSend by Cronos\n");
		printf("Mailserver :%s\n",argv[mailnum]);
		printf("Textfile   :%s\n",argv[textnum]);
	}

	if(init_winsock(MAKEWORD(1, 1),verb)) exit(-1);

	if(verb) printf("Looking up mailserver\n");
	padr=hostlookupaddr(argv[mailnum],verb);
	if(!padr)
	{	exit_winsock();
		exit(-1);
	}

	if(getsocket(&mailsock,verb))
	{ 	exit_winsock();
		exit(-1);
	}

	if(!opensocket(mailsock,padr,SMTP_PORT,verb))
	{  if(getresponse(mailsock,R,MAILBUFFSIZE)>=0)
		{	sendline(mailsock,"HELO\r\n");
			if(getresponse(mailsock,R,MAILBUFFSIZE)>=0)
				sendfile(mailsock,argv[textnum]);
			sendline(mailsock,"QUIT\r\n");
			if(getresponse(mailsock,R,MAILBUFFSIZE))
			{	sendline(mailsock,".\r\n");
				sendline(mailsock,"QUIT\r\n");
			}
			getresponse(mailsock,R,MAILBUFFSIZE);
		}
		closesocket(mailsock);
	}

	exit_winsock();
	return 0;
}
